﻿using DatabaseExtension;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using DatabaseExtension.ConnectionString;
using System.Linq;
using Oracle.DataAccess.Client;
using SQLiteProvider.ConnectionString;

namespace Test
{
    
    
    /// <summary>
    ///Dies ist eine Testklasse für "DbInterfaceTest" und soll
    ///alle DbInterfaceTest Komponententests enthalten.
    ///</summary>
    [TestClass()]
    public class DbInterfaceTest
    {


        private TestContext testContextInstance;

        /// <summary>
        ///Ruft den Testkontext auf, der Informationen
        ///über und Funktionalität für den aktuellen Testlauf bietet, oder legt diesen fest.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Zusätzliche Testattribute
        // 
        //Sie können beim Verfassen Ihrer Tests die folgenden zusätzlichen Attribute verwenden:
        //
        //Mit ClassInitialize führen Sie Code aus, bevor Sie den ersten Test in der Klasse ausführen.
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Mit ClassCleanup führen Sie Code aus, nachdem alle Tests in einer Klasse ausgeführt wurden.
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Mit TestInitialize können Sie vor jedem einzelnen Test Code ausführen.
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Mit TestCleanup können Sie nach jedem einzelnen Test Code ausführen.
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion


        /// <summary>
        ///Test Sql Compact edition connection. Create and delete db.
        ///</summary>
        [TestMethod()]
        public void DbInterfaceSqlCeTest()
        {
            String server = System.IO.Path.Combine(Environment.CurrentDirectory, "tmp.sdf");
            int expected = 1;
            int actual = -1;
            using (DbInterface target = new DbInterface(new SqlCeConnectionInfo(server, "test") { FailIfMissing = false }))
            {
                actual = (int)target.SqlExecuteScalar("SELECT 1");
            }
            System.IO.File.Delete(server);
            Assert.AreEqual(expected, actual);
        }
        /// <summary>
        ///Test Sql Compact edition connection. Create and delete db with parameter.
        ///</summary>
        [TestMethod()]
        public void DbInterfaceSqlCeParamTest()
        {
            String server = System.IO.Path.Combine(Environment.CurrentDirectory, "tmp.sdf");
            int expected = 1;
            int actual = -1;
            using (DbInterface target = new DbInterface(new SqlCeConnectionInfo(server, "test") { FailIfMissing = false }))
            {
                target.SetSqlParameter("@test", 1);
                actual = (int)target.SqlExecuteScalar("SELECT @test");
            }
            System.IO.File.Delete(server);
            Assert.AreEqual(expected, actual);
        }
        /// <summary>
        ///Test Sql Compact edition connection. Create and delete db with parameter.
        ///</summary>
        [TestMethod()]
        public void DbInterfaceSqlCeParamStringTest()
        {
            String server = System.IO.Path.Combine(Environment.CurrentDirectory, "tmp.sdf");
            String  expected = "te",
                    actual = String.Empty;
            using (DbInterface target = new DbInterface(new SqlCeConnectionInfo(server, "test") { FailIfMissing = false }))
            {
                target.SetSqlParameter("@test", "test", 2);
                actual = target.SqlExecuteScalar("SELECT @test").ToString();
            }
            System.IO.File.Delete(server);
            Assert.AreEqual(expected, actual);
        }
        /// <summary>
        ///Test SQlite connection. Create in memory
        ///</summary>
        [TestMethod()]
        public void DbInterfaceSQLiteMemTest()
        {
            String server = ":memory:";
            long expected = 1;
            long actual = -1;
            using (DbInterface target = new DbInterface(new SQLiteConnectionInfo(server, "test") { FailIfMissing = false }))
            {
                actual = (long)target.SqlExecuteScalar("SELECT 1");
            }
            Assert.AreEqual(expected, actual);
        }
        /// <summary>
        ///Test SQlite connection. Create in memory
        ///</summary>
        [TestMethod()]
        public void DbInterfaceSQLiteMemParamTest()
        {
            String server = ":memory:";
            long expected = 1;
            long actual = -1;
            using (DbInterface target = new DbInterface(new SQLiteConnectionInfo(server, "test") { FailIfMissing = false }))
            {
                target.SetSqlParameter("@test", expected);
                actual = (long)target.SqlExecuteScalar("SELECT @test");
            }
            Assert.AreEqual(expected, actual);
        }
        /// <summary>
        ///Test SQlite connection. Create in memory
        ///</summary>
        [TestMethod()]
        public void DbInterfaceSQLiteMemParamStringTest()
        {
            String server = ":memory:";
            String expected = "test";
            String actual = String.Empty;
            using (DbInterface target = new DbInterface(new SQLiteConnectionInfo(server, "test") { FailIfMissing = false }))
            {
                target.SetSqlParameter("@test", expected);
                actual = target.SqlExecuteScalar("SELECT @test").ToString();
            }
            Assert.AreEqual(expected, actual);
        }
        /// <summary>
        ///Test SQlite connection. Create and delete db.
        ///</summary>
        [TestMethod()]
        public void DbInterfaceSQLiteTest()
        {
            String server = System.IO.Path.Combine(Environment.CurrentDirectory, "tmp.sdf");
            long expected = 1;
            long actual = -1;
            using (DbInterface target = new DbInterface(new SQLiteConnectionInfo(server, "test") { FailIfMissing = false }))
            {
                actual = (long)target.SqlExecuteScalar("SELECT 1");
            }
            System.IO.File.Delete(server);
            Assert.AreEqual(expected, actual);
        }
        /// <summary>
        ///Test MSSQL connection. SELECT 1
        ///</summary>
        [TestMethod()]
        public void DbInterfaceMSSQLTest()
        {
            long expected = 1;
            long actual = -1;
            using (DbInterface target = new DbInterface(new SqlConnectionInfo("localhost") { IntegratedSecurity = true}))
            {
                actual = (int)target.SqlExecuteScalar("SELECT 1");
            }
            Assert.AreEqual(expected, actual);
        }
        /// <summary>
        ///Test MSSQL connection. SELECT 1
        ///</summary>
        [TestMethod()]
        public void DbInterfaceMSSQLParamTest()
        {
            long expected = 1;
            long actual = -1;
            using (DbInterface target = new DbInterface(new SqlConnectionInfo("localhost") { IntegratedSecurity = true }))
            {
                target.SetSqlParameter("@test", 1);
                actual = (int)target.SqlExecuteScalar("SELECT @test");
            }
            Assert.AreEqual(expected, actual);
        }
        /// <summary>
        ///Test OleDb on Microsoft connection. SELECT 1
        ///</summary>
        [TestMethod()]
        public void DbInterfaceOleDbToMSSQL2008Test()
        {
            long expected = 1;
            long actual = -1;
            var con = new OleDbConnectionInfo("localhost", null, null) { Provider = "SQLNCLI10" };
            con.OleDbConnectionBuilder.Add("Trusted_Connection", "yes");
            con.OleDbConnectionBuilder.Add("Server", "localhost");
            using (DbInterface target = new DbInterface(con))
            {
                actual = (int)target.SqlExecuteScalar("SELECT 1");
            }
            Assert.AreEqual(expected, actual);
        }
        /// <summary>
        ///Test OleDb on Microsoft connection. SELECT 1
        ///</summary>
        [TestMethod()]
        public void DbInterfaceOleDbToMSSQL2008ParamTest()
        {
            long expected = 1;
            long actual = -1;
            var con = new OleDbConnectionInfo("localhost", null, null) { Provider = "SQLNCLI10" };
            con.OleDbConnectionBuilder.Add("Trusted_Connection", "yes");
            con.OleDbConnectionBuilder.Add("Server", "localhost");
            using (DbInterface target = new DbInterface(con))
            {
                target.SetSqlParameter("?", 1);
                actual = (int)target.SqlExecuteScalar("SELECT ?");
            }
            Assert.AreEqual(expected, actual);
        }
    }
}
